#!/usr/bin/env python

"""
Read JSON module info. Write formatted asciidoc.

usage:
  format_to_asciidoc.py [-h] [-f] inputfile [outputfile]

positional arguments:
  inputfile    The JSON input file to be read. Use "-" to read stdin.
  outputfile   The name of the file to write to. Default: sys.stdout.

optional arguments:
  -h, --help   show this help message and exit
  -f, --force  Overwrite output file without asking.
"""

from __future__ import print_function
import sys
import os
import argparse
import json


def format_to_asciidoc(info, outputfile):
    wrt = outputfile.write
    module_info = info['module']
    wrt('= Module information for: {}\n\n'.format(module_info['name']))
    wrt('== Module documentation\n\n')
    wrt('..........\n')
    wrt('{}\n'.format(module_info['doc']))
    wrt('..........\n\n')
    wrt('== Imports\n\n')
    import_info_collection = info['imports']
    for import_info in import_info_collection:
        wrt('- Name: {}  Path: {}\n\n'.format(
            import_info['name'],
            import_info['path']))
    wrt('== Classes\n')
    class_info_collection = info['classes']
    for class_info in class_info_collection:
        wrt('\n- Class name: {}\n'.format(class_info['name']))
        wrt('+\nInstance methods:\n')
        instance_method_info_collection = class_info['methods']
        for method_info in instance_method_info_collection:
            wrt('+\n* Name: {}\n'.format(method_info['name']))
            wrt('+\nDoc:\n+\n..........')
            wrt('\n{}\n'.format(method_info['doc']))
            wrt('..........\n')
            wrt('+\nSignature: {}\n'.format(method_info['sig']))
    wrt('\n== Functions\n')
    function_info_collection = info['functions']
    for function_info in function_info_collection:
        wrt('\n- Name: {}\n'.format(function_info['name']))
        wrt('+\nDoc:\n+\n..........')
        wrt('\n{}\n'.format(function_info['doc']))
        wrt('..........\n')
        wrt('+\nSignature: {}\n'.format(function_info['sig']))
    wrt('\n// vim:ft=asciidoc:\n')


def makefile(options):
    if os.path.exists(options.outputfile) and not options.force:
        sys.exit('Output file exists.  Use "-f" to overwrite.')
    outfile = open(options.outputfile, 'w')
    return outfile


def format_and_write(options):
    if options.outputfile:
        outputfile = makefile(options)
    else:
        outputfile = sys.stdout
    with (sys.stdin if options.inputfile == '-' else
            open(options.inputfile, 'r')) as inputfile:
        json_info = json.load(inputfile)
        format_to_asciidoc(json_info, outputfile)
        if options.outputfile:
            outputfile.close()


def main():
    desc = 'Read JSON module info.  Write formatted asciidoc.'
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument(
        'inputfile',
        help='The JSON input file to be read.  Use "-" to read stdin.')
    parser.add_argument(
        'outputfile',
        nargs='?', default='',
        help='The name of the file to write to.  Default: sys.stdout.')
    parser.add_argument(
        '-f', '--force', action='store_true',
        help='Overwrite output file without asking.')
    options = parser.parse_args()
    format_and_write(options)


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()
